// Verkaufsautomat mit Wechselgeldrueckgabe und Abbruchfunktion
`timescale 1 ns / 1 ps
module iglu_multicoin_change (
  input wire       CLOCK,    // Takt
                   RESET,    // Reset
                   CANCEL,   // Abbruch mit Geldrueckgabe
  input wire [2:0] COIN,     // eingeworfene Muenze
  output reg       ICE,      // Warenausgabe
  output reg [2:0] OUTCOIN); // herauszugebene Muenze

  `include "iglu_multicoin_change_defs.v"

  // interne Variable
  reg [1:0] PRESENT,      // jetziger Zustand
            NEXT;         // naechster Zustand

  reg [5:0] SUM;          // eingezahlter Geldbetrag
                          // Einheit 10 Cent
  reg [5:0] NEXT_SUM;     // neue Summe
                          // Einheit 10 Cent

  always @(PRESENT, COIN, SUM, CANCEL)// <-- flankenfrei!
    case (PRESENT)
      INSERT_COINS: begin             // Muenzeinwurf
        case (COIN)
          X0:                         // kein Einwurf
            NEXT_SUM = SUM + 0;
          X10:                        // 10 Cent
            NEXT_SUM = SUM + 1;
          X20:                        // 20 Cent
            NEXT_SUM = SUM + 2;
          X50:                        // 50 Cent
            NEXT_SUM = SUM + 5;
          X100:                       // 1 Euro
            NEXT_SUM = SUM + 10;
          X200:                       // 2 Euro
            NEXT_SUM = SUM + 20;
          default:                    // Don't care, Latch vermeiden
            NEXT_SUM = 'bx;
        endcase
        if (SUM >= Price)             // Eis ausgeben mit Wechselgeld
          {NEXT,ICE,OUTCOIN} = {ICECREAM,No,X0};
        else if (CANCEL)              // Abbruch durch Kunden, Geldrueckgabe
          {NEXT,ICE,OUTCOIN} = {CHANGE,No,X0};
        else
          {NEXT,ICE,OUTCOIN} = {INSERT_COINS,No,X0};
      end
      ICECREAM: begin                 // genug bezahlt
        NEXT_SUM = SUM - Price;       // Restgeldbetrag
        {NEXT,ICE,OUTCOIN} =          // Warenausgabe, Wechselgeld
          {CHANGE,Yes,X0};
      end
      CHANGE: begin                   // Wechselgeld ausgeben, Greedy
        if (SUM >= 10) begin          // ein Euro ausgeben
          NEXT_SUM = SUM - 10;
          {NEXT,ICE,OUTCOIN} = {WAIT,No,X100};
        end
        else if (SUM >= 5) begin      // 50 Cent
          NEXT_SUM = SUM - 5;
          {NEXT,ICE,OUTCOIN} = {WAIT,No,X50};
        end
        else if (SUM >= 2) begin      // 20 Cent
          NEXT_SUM = SUM - 2;
          {NEXT,ICE,OUTCOIN} = {WAIT,No,X20};
        end
        else if (SUM == 1) begin      // 10 Cent
          NEXT_SUM = SUM - 1;
          {NEXT,ICE,OUTCOIN} = {WAIT,No,X10};
        end
        else begin                    // Alles Wechselgeld ausgegeben
          NEXT_SUM = SUM;
          {NEXT,ICE,OUTCOIN} = {INSERT_COINS,No,X0};
        end
      end
      WAIT: begin                     // Pause zwischen Muenzausgaben
        NEXT_SUM = SUM;
        {NEXT,ICE,OUTCOIN} = {CHANGE,No,X0};
      end
    endcase

  // naechsten Zustand mit jedem Takt endgueltig zum jetzigen machen;
  // synchrones Reset
  //
  always @(posedge CLOCK)             // <-- flankengesteuert!
    if (RESET == 1'b1) begin
      PRESENT <= INSERT_COINS;        // Anfangszustand
      SUM <= 0;
    end
    else begin
      PRESENT <= NEXT;
      SUM <= NEXT_SUM;
    end

endmodule // iglu_multicoin_change
